#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import functools
import re
import os

from requests import Session
from bs4 import BeautifulSoup as _Bs
from bs4 import Tag, NavigableString

try:
    __import__('lxml')
    BeautifulSoup = lambda makeup: _Bs(makeup, 'lxml')
except ImportError:
    BeautifulSoup = lambda makeup: _Bs(makeup, 'html.parser')

Default_Header = {'X-Requested-With': 'XMLHttpRequest',
                  'Referer': 'http://www.zhihu.com',
                  'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; '
                                'rv:39.0) Gecko/20100101 Firefox/39.0',
                  'Host': 'www.zhihu.com'}

Zhihu_URL = 'https://www.zhihu.com'
Login_URL = Zhihu_URL + '/login/email'
Captcha_URL_Prefix = Zhihu_URL + '/captcha.gif?r='
Get_Profile_Card_URL = Zhihu_URL + '/node/MemberProfileCardV2'
Question_Get_More_Answer_URL = Zhihu_URL + '/node/QuestionAnswerListV2'
Answer_Add_Comment_URL = Zhihu_URL + '/node/AnswerCommentAddV2'
Answer_Comment_Box_URL = Zhihu_URL + '/node/AnswerCommentBoxV2'
Author_Get_More_Followers_URL = Zhihu_URL + '/node/ProfileFollowersListV2'
Author_Get_More_Followees_URL = Zhihu_URL + '/node/ProfileFolloweesListV2'
Author_Get_More_Follow_Column_URL = Zhihu_URL + '/node/ProfileFollowedColumnsListV2'
Author_Get_More_Follow_Topic_URL = Zhihu_URL + '/people/{0}/topics'

PROTOCOL = ''

Column_Url = 'http://zhuanlan.zhihu.com'
Column_API = Column_Url + '/api/columns'
Column_Data = Column_API + '/{0}'
Column_Posts_Data = Column_API + '/{0}/posts?limit=10&offset={1}'
Column_Post_Data = Column_API + '/{0}/posts/{1}'

Topic_Url = Zhihu_URL + '/topic'
Topic_Get_Children_Url = Topic_Url + '/{0}/organize/entire'
Topic_Get_More_Follower_Url = Topic_Url + '/{0}/followers'
Topic_Question_Url = Topic_Url + '/{0}/questions'
Topic_Top_Answers_Url = Topic_Url + '/{0}/top-answers'
Topic_Hot_Questions_Url = Topic_Url + '/{0}/hot'
Topic_Newest_Url = Topic_Url + '/{0}/newest'

Get_Me_Info_Url = Column_Url + '/api/me'
Upvote_Answer_Url = Zhihu_URL + '/node/AnswerVoteBarV2'
Upvote_Article_Url = Column_API + '/{0}/posts/{1}/rating'
Follow_Author_Url = Zhihu_URL + '/node/MemberFollowBaseV2'
Follow_Question_Url = Zhihu_URL + '/node/QuestionFollowBaseV2'
Follow_Topic_Url = Zhihu_URL + '/node/TopicFollowBaseV2'
Follow_Collection_Url = Zhihu_URL + '/collection/follow'
Unfollow_Collection_Url = Zhihu_URL + '/collection/unfollow'
Thanks_Url = Zhihu_URL + '/answer/thanks'
Cancel_Thanks_Url = Zhihu_URL + '/answer/cancel_thanks'
Send_Message_Url = Zhihu_URL + '/inbox/post'
Unhelpful_Url = Zhihu_URL + '/answer/not_helpful'
Cancel_Unhelpful_Url = Zhihu_URL + '/answer/helpful'

re_question_url = re.compile(r'^https?://www\.zhihu\.com/question/\d+/?$')
re_ans_url = re.compile(
    r'^https?://www\.zhihu\.com/question/\d+/answer/\d+/?$')
re_author_url = re.compile(r'^https?://www\.zhihu\.com/people/[^/]+/?$')
re_collection_url = re.compile(r'^https?://www\.zhihu\.com/collection/\d+/?$')
re_column_url = re.compile(r'^http://zhuanlan\.zhihu\.com/([^/]+)/?$')
re_post_url = re.compile(r'^http://zhuanlan\.zhihu\.com/([^/]+)/(\d+)/?$')
re_topic_url = re.compile(r'^https?://www\.zhihu\.com/topic/(\d+)/?$')
re_a2q = re.compile(r'(.*)/a.*')
re_collection_url_split = re.compile(r'.*(/c.*)')
re_get_number = re.compile(r'[^\d]*(\d+).*')
re_del_empty_line = re.compile(r'\n*(.*)\n*')


def check_soup(attr, soup_type='_make_soup'):
    def real(func):
        @functools.wraps(func)
        def wrapper(self):
            # noinspection PyTypeChecker
            value = getattr(self, attr, None)
            if value is None:
                if soup_type == '_make_soup':
                    getattr(self, soup_type)()
                elif self.soup is None:
                    getattr(self, soup_type)()
                value = func(self)
                setattr(self, attr, value)
            return value

        return wrapper

    return real


def class_common_init(url_re, allowed_none=True):
    def real(func):
        @functools.wraps(func)
        def wrapper(self, url, *args, **kwargs):
            if url is None and not allowed_none:
                raise ValueError('Invalid Url: ' + url)
            if url is not None:
                if url_re.match(url) is None:
                    raise ValueError('Invalid URL: ' + url)
                if url.endswith('/') is False:
                    url += '/'
            if 'session' not in kwargs.keys() or kwargs['session'] is None:
                kwargs['session'] = Session()
            self.soup = None
            return func(self, url, *args, **kwargs)

        return wrapper

    return real


def remove_invalid_char(text):
    """去除字符串中的无效字符，一般用于保存文件时保证文件名的有效性.

    :param str text: 待处理的字符串
    :return: 处理后的字符串
    :rtype: str
    """
    invalid_char_list = ['/', '\\', ':', '*', '?', '"', '<', '>', '|', '\n']
    res = ''
    for char in text:
        if char not in invalid_char_list:
            res += char
    return res


def parser_author_from_tag(author):
    author_link = author.find('a', class_='author-link')
    if author_link is None:
        return None, '匿名用户', '', ''
    else:
        author_name = author_link.text
        motto_span = author.find('span', class_='bio')
        author_motto = motto_span['title'] \
            if motto_span is not None else ''
        author_url = Zhihu_URL + author_link['href']
        avatar_link = author.find('a', class_='avatar-link')
        photo_url = PROTOCOL + avatar_link.img['src'].replace('_s', '_r')
        return author_url, author_name, author_motto, photo_url


def parser_author_from_comment(author):
    author_avatar = author.find('a', class_='zm-item-link-avatar')
    if author_avatar is None:
        return None, '匿名用户', ''
    else:
        author_link = author.find('a', class_='zg-link')
        author_name = author_link.text
        author_url = author_link['href']
        avatar_link = author.find('img', class_='zm-item-img-avatar')
        photo_url = PROTOCOL + avatar_link['src'].replace('_s', '_r')
        return author_url, author_name,  photo_url


def answer_content_process(content):
    content = clone_bs4_elem(content)
    del content['class']
    soup = BeautifulSoup(
        '<html><head><meta charset="utf-8"></head><body></body></html>')
    soup.body.append(content)
    no_script_list = soup.find_all("noscript")
    for no_script in no_script_list:
        no_script.extract()
    img_list = soup.find_all(
        "img", class_=["origin_image", "content_image"])
    for img in img_list:
        if "content_image" in img['class']:
            img['data-original'] = img['data-actualsrc']
        new_img = soup.new_tag('img', src=PROTOCOL + img['data-original'])
        img.replace_with(new_img)
        if img.next_sibling is None:
            new_img.insert_after(soup.new_tag('br'))
    useless_list = soup.find_all("i", class_="icon-external")
    for useless in useless_list:
        useless.extract()
    return soup.prettify()


def get_path(path, filename, mode, default_path, default_name):
    if path is None:
        path = os.path.join(
            os.getcwd(), remove_invalid_char(default_path))
    if filename is None:
        filename = remove_invalid_char(default_name)
    if os.path.isdir(path) is False:
        os.makedirs(path)
    temp = filename
    i = 0
    while os.path.isfile(os.path.join(path, temp) + '.' + mode):
        i += 1
        temp = filename + str(i)
    return os.path.join(path, temp) + '.' + mode


def common_follower(url, xsrf, session):
    from .author import Author
    headers = dict(Default_Header)
    headers['Referer'] = url
    data = {'offset': 0, '_xsrf': xsrf}
    gotten_data_num = 20
    offset = 0
    while gotten_data_num == 20:
        data['offset'] = offset
        res = session.post(url, data=data, headers=headers)
        json_data = res.json()['msg']
        gotten_data_num = json_data[0]
        offset += gotten_data_num
        soup = BeautifulSoup(json_data[1])
        follower_divs = soup.find_all('div', class_='zm-profile-card')
        for div in follower_divs:
            if div.a is not None:
                author_name = div.a['title']
                author_url = Zhihu_URL + div.a['href']
                author_motto = div.find('div', class_='zg-big-gray').text
                author_photo = PROTOCOL + div.img['src'].replace('_m', '_r')
                numbers = [re_get_number.match(a.text).group(1)
                           for a in div.find_all('a', target='_blank')]
            else:
                author_name = '匿名用户'
                author_url = None
                author_motto = ''
                author_photo = None
                numbers = [None] * 4
            yield Author(author_url, author_name, author_motto, *numbers,
                         photo_url=author_photo, session=session)


def clone_bs4_elem(el):
    """Clone a bs4 tag before modifying it.

    Code from `http://stackoverflow.com/questions/23057631/clone-element-with
    -beautifulsoup`
    """
    if isinstance(el, NavigableString):
        return type(el)(el)

    copy = Tag(None, el.builder, el.name, el.namespace, el.nsprefix)
    # work around bug where there is no builder set
    # https://bugs.launchpad.net/beautifulsoup/+bug/1307471
    copy.attrs = dict(el.attrs)
    for attr in ('can_be_empty_element', 'hidden'):
        setattr(copy, attr, getattr(el, attr))
    for child in el.contents:
        copy.append(clone_bs4_elem(child))
    return copy



#!/usr/bin/env python3
# -*- coding: utf-8 -*-


class Comment:

    """评论类，一般不直接使用，而是作为``Answer.comments``迭代器的返回类型."""

    def __init__(self, cid, answer, author,
                 upvote_num, content, time_string, group_id=None):
        """创建评论类实例.

        :param int cid: 评论ID
        :param int group_id: 评论所在的组ID
        :param Answer answer: 评论所在的答案对象
        :param Author author: 评论的作者对象
        :param int upvote_num: 评论赞同数量
        :param str content: 评论内容
        :param str time_string: 表示评论时间的字符串
        :return: 评论对象
        :rtype: Comment
        """

        self.cid = cid
        self.answer = answer
        self.author = author
        self.upvote_num = upvote_num
        self.content = content
        self.time_string = time_string
        self._group_id = group_id



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

__author__ = '7sDream'
__version__ = '0.3.4'

from .client import ZhihuClient
from .question import Question
from .author import Author
from .activity import Activity
from .acttype import ActType
from .answer import Answer
from .collection import Collection
from .column import Column
from .post import Post
from .topic import Topic

__all__ = ['ZhihuClient', 'Question', 'Author', 'ActType', 'Activity',
           'Answer', 'Collection', 'Column', 'Post', 'Topic']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os

from test_utils import TEST_DATA_PATH
from zhihu import Question, Author, Answer
from zhihu.common import BeautifulSoup


description = ("从小父母和大家庭里，长辈都教我们得到别人帮助时要说“谢谢”。"
               "比方说家庭聚餐，亲人们帮忙夹了菜要感谢。无论多么亲密,父母还是"
               "兄妹，都要说声谢谢。   后来上了高中，大学。也习惯性的及时表达"
               "对他人帮助的感谢。  但室友们，还有男朋友，都不喜欢我这样。他们"
               "说，这样说的话会感觉双方很有距离感很生疏。尤其是男朋友，不喜"
               "欢我这样，他说这样很不亲密，情侣之间就不分得那么清，不需要谢"
               "谢的。但我从小习惯了，别人帮了我我不说的话，会很不自在。  "
               '怎么办？我还要继续这样吗？什么时候不该说"谢谢”？？')


class QuestionTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://www.zhihu.com/question/24825703'
        file_path = os.path.join(TEST_DATA_PATH, 'question.html')
        with open(file_path, 'rb') as f:
            html = f.read()
        soup = BeautifulSoup(html)

        cls.question = Question(url)
        cls.question._session = None
        cls.question.soup = soup
        cls.expected = {'id': 24825703, 'qid': 2112271,
                        'xsrf': 'cfd489623d34ca03adfdc125368c6426',
                        'html': soup.prettify(),
                        'title': '关系亲密的人之间要说「谢谢」吗？',
                        'details': description, 'answer_num': 621,
                        'follower_num': 4427, 'top_answer_id': 39753456,
                        'top_answer_author_name': '芝士就是力量',
                        'top_answer_upvote_num': 97, 'top_50_ans_id': 31003847,
                        'top_50_ans_author_name': '圭多达莱佐',
                        'top_50_ans_upvote_num': 31, 'more_ans_id': 39958704,
                        'more_ans_author_name': '柳蜻蜓',
                        'more_ans_upvote_num': 1,
                        'topics': ['心理学', '恋爱', '社会', '礼仪',
                                   '亲密关系'],
                        }

        more_ans_file_path = os.path.join(TEST_DATA_PATH,
                                          'question_more_answer.html')
        with open(more_ans_file_path, 'rb') as f:
            cls.more_ans_html = f.read()

    def test_id(self):
        self.assertEqual(self.expected['id'], self.question.id)

    def test_qid(self):
        self.assertEqual(self.expected['qid'], self.question.qid)

    def test_xsrf(self):
        self.assertEqual(self.expected['xsrf'], self.question.xsrf)

    def test_html(self):
        self.assertEqual(self.expected['html'], self.question.html)

    def test_title(self):
        self.assertEqual(self.expected['title'], self.question.title)

    def test_details(self):
        self.assertEqual(self.expected['details'], self.question.details)

    def test_answer_num(self):
        self.assertEqual(self.expected['answer_num'], self.question.answer_num)

    def test_follower_num(self):
        self.assertEqual(self.expected['follower_num'],
                         self.question.follower_num)

    def test_topics(self):
        self.assertEqual(self.expected['topics'], self.question.topics)

    def test_top_answer(self):
        answer = self.question.top_answer
        self.assertEqual(self.expected['top_answer_id'], answer.id)
        self.assertEqual(self.expected['top_answer_author_name'],
                         answer.author.name)
        self.assertEqual(self.expected['top_answer_upvote_num'],
                         answer.upvote_num)

    def test_top_i_answer(self):
        answer = self.question.top_i_answer(50)
        self.assertEqual(self.expected['top_50_ans_id'], answer.id)
        self.assertEqual(self.expected['top_50_ans_author_name'],
                         answer.author.name)
        self.assertEqual(self.expected['top_50_ans_upvote_num'],
                         answer.upvote_num)

    def test_parse_answer_html(self):
        answer = self.question._parse_answer_html(self.more_ans_html, Author,
                                                  Answer)
        self.assertEqual(self.expected['more_ans_id'], answer.id)
        self.assertEqual(self.expected['more_ans_author_name'],
                         answer.author.name)
        self.assertEqual(self.expected['more_ans_upvote_num'],
                         answer.upvote_num)

    def test_top_i_answers(self):
        answers = [a for a in self.question.top_i_answers(1)]
        answer = answers[0]
        self.assertEqual(self.expected['top_answer_id'], answer.id)
        self.assertEqual(self.expected['top_answer_author_name'],
                         answer.author.name)
        self.assertEqual(self.expected['top_answer_upvote_num'],
                         answer.upvote_num)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from .common import *
from .base import BaseZhihu
import time


class Topic(BaseZhihu):

    """答案类，请使用``ZhihuClient.topic``方法构造对象."""

    @class_common_init(re_topic_url)
    def __init__(self, url, name=None, session=None):
        """创建话题类实例.

        :param url: 话题url
        :param name: 话题名称，可选
        :return: Topic
        """
        self.url = url
        self._session = session
        self._name = name
        self._id = int(re_topic_url.match(self.url).group(1))

    @property
    def id(self):
        """获取话题Id（网址最后那串数字）

        :return: 话题Id
        :rtype: int
        """
        return self._id

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        return self.soup.find('input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_tid')
    def tid(self):
        """话题内部Id，有时候要用到

        :return: 话题内部Id
        :rtype: int
        """
        return int(self.soup.find(
            'div', id='zh-topic-desc')['data-resourceid'])

    @property
    @check_soup('_name')
    def name(self):
        """获取话题名称.

        :return: 话题名称
        :rtype: str
        """
        return self.soup.find('h1').text

    @property
    def parents(self):
        """获取此话题的父话题。
        注意：由于没找到有很多父话题的话题来测试，
        所以本方法可能再某些时候出现问题，请不吝反馈。

        :return: 此话题的父话题，返回生成器
        :rtype: Topic.Iterable
        """
        self._make_soup()
        parent_topic_tag = self.soup.find('div', class_='parent-topic')
        if parent_topic_tag is None:
            return []
        else:
            for topic_tag in parent_topic_tag.find_all('a'):
                yield Topic(Zhihu_URL + topic_tag['href'],
                            topic_tag.text.strip(),
                            session=self._session)

    @property
    def children(self):
        """获取此话题的子话题

        :return: 此话题的子话题， 返回生成器
        :rtype: Topic.Iterable
        """
        self._make_soup()
        child_topic_tag = self.soup.find('div', class_='child-topic')
        if child_topic_tag is None:
            return []
        elif '共有' not in child_topic_tag.contents[-2].text:
            for topic_tag in child_topic_tag.div.find_all('a'):
                yield Topic(Zhihu_URL + topic_tag['href'],
                            topic_tag.text.strip(),
                            session=self._session)
        else:
            flag = 'load'
            child = ''
            data = {'_xsrf': self.xsrf}
            params = {
                'parent': self.id
            }
            while flag == 'load':
                params['child'] = child
                res = self._session.post(Topic_Get_Children_Url,
                                         params=params, data=data)
                j = map(lambda x: x[0], res.json()['msg'][1])
                *topics, last = j
                for topic in topics:
                    yield Topic(Zhihu_URL + '/topic/' + topic[2], topic[1],
                                session=self._session)
                flag = last[0]
                child = last[2]
                if flag == 'topic':
                    yield Topic(Zhihu_URL + '/topic/' + last[2], last[1],
                                session=self._session)

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取话题关注人数.

        :return: 关注人数
        :rtype: int
        """
        follower_num_block = self.soup.find(
            'div', class_='zm-topic-side-followers-info')
        # 无人关注时 找不到对应block，直接返回0 （感谢知乎用户 段晓晨 提出此问题）
        if follower_num_block.strong is None:
            return 0
        return int(follower_num_block.strong.text)

    @property
    def followers(self):
        """获取话题关注者

        :return: 话题关注者，返回生成器
        :rtype: Author.Iterable
        """
        from .author import Author
        self._make_soup()
        gotten_data_num = 20
        data = {
            '_xsrf': self.xsrf,
            'start': '',
            'offset': 0
        }
        while gotten_data_num == 20:
            res = self._session.post(
                Topic_Get_More_Follower_Url.format(self.id), data=data)
            j = res.json()['msg']
            gotten_data_num = j[0]
            data['offset'] += gotten_data_num
            soup = BeautifulSoup(j[1])
            divs = soup.find_all('div', class_='zm-person-item')
            for div in divs:
                h2 = div.h2
                url = Zhihu_URL + h2.a['href']
                name = h2.a.text
                motto = h2.next_element.text
                yield Author(url, name, motto, session=self._session)
            data['start'] = int(re_get_number.match(divs[-1]['id']).group(1))

    @property
    @check_soup('_photo_url')
    def photo_url(self):
        """获取话题头像图片地址.

        :return: 话题头像url
        :rtype: str
        """
        img = self.soup.find('a', id='zh-avartar-edit-form').img['src']
        return img.replace('_m', '_r')

    @property
    @check_soup('_description')
    def description(self):
        """获取话题描述信息.

        :return: 话题描述信息
        :rtype: str
        """
        desc = self.soup.find('div', class_='zm-editable-content').text
        return desc

    @property
    def top_authors(self):
        """获取最佳回答者

        :return: 此话题下最佳回答者，一般来说是5个，要不就没有，返回生成器
        :rtype: Author.Iterable
        """
        from .author import Author
        self._make_soup()
        t = self.soup.find('div', id='zh-topic-top-answerer')
        if t is None:
            return
        for d in t.find_all('div', class_='zm-topic-side-person-item-content'):
            url = Zhihu_URL + d.a['href']
            name = d.a.text
            motto = d.div['title']
            yield Author(url, name, motto, session=self._session)

    @property
    def top_answers(self):
        """获取话题下的精华答案.

        :return: 话题下的精华答案，返回生成器.
        :rtype: Answer.Iterable
        """
        from .question import Question
        from .answer import Answer
        from .author import Author
        top_answers_url = Topic_Top_Answers_Url.format(self.id)
        params = {'page': 1}
        while True:
            # 超出50页直接返回
            if params['page'] > 50:
                return
            res = self._session.get(top_answers_url, params=params)
            params['page'] += 1
            soup = BeautifulSoup(res.content)
            # 不够50页，来到错误页面 返回
            if soup.find('div', class_='error') is not None:
                return
            questions = soup.find_all('a', class_='question_link')
            answers = soup.find_all(
                'a', class_=re.compile(r'answer-date-link.*'))
            authors = soup.find_all('h3', class_='zm-item-answer-author-wrap')
            upvotes = soup.find_all('a', class_='zm-item-vote-count')
            for ans, up, q, au in zip(answers, upvotes, questions, authors):
                answer_url = Zhihu_URL + ans['href']
                question_url = Zhihu_URL + q['href']
                question_title = q.text
                upvote = int(up['data-votecount'])
                question = Question(question_url, question_title,
                                    session=self._session)
                if au.a is None:
                    author_url = None
                    author_name = '匿名用户'
                    author_motto = ''
                else:
                    author_url = Zhihu_URL + au.a['href']
                    author_name = au.a.text
                    author_motto = au.strong['title'] if au.strong else ''

                author = Author(author_url, author_name, author_motto,
                                session=self._session)
                yield Answer(answer_url, question, author, upvote,
                             session=self._session)

    @property
    def questions(self):
        """获取话题下的所有问题（按时间降序排列）

        :return: 话题下的所有问题，返回生成器
        :rtype: Question.Iterable
        """
        from .question import Question
        question_url = Topic_Question_Url.format(self.id)
        params = {'page': 1}
        older_time_stamp = int(time.time()) * 1000
        while True:
            res = self._session.get(question_url, params=params)
            soup = BeautifulSoup(res.content)
            if soup.find('div', class_='error') is not None:
                return
            questions = soup.find_all('div', class_='question-item')
            questions = list(filter(
                lambda x: int(x.h2.span['data-timestamp']) < older_time_stamp,
                questions))
            for qu_div in questions:
                url = Zhihu_URL + qu_div.h2.a['href']
                title = qu_div.h2.a.text
                yield Question(url, title, session=self._session)
            older_time_stamp = int(questions[-1].h2.span['data-timestamp'])
            params['page'] += 1

    @property
    def answers(self):
        """获取话题下所有答案（按时间降序排列）

        :return: 话题下所有答案，返回生成器
        :rtype: Answer.Iterable
        """
        from .question import Question
        from .answer import Answer
        from .author import Author
        newest_url = Topic_Newest_Url.format(self.id)
        params = {'start': 0, '_xsrf': self.xsrf}
        res = self._session.get(newest_url)
        soup = BeautifulSoup(res.content)
        while True:
            divs = soup.find_all('div', class_='folding')
            # 如果话题下无答案，则直接返回
            if len(divs) == 0:
                return
            last_score = divs[-1]['data-score']
            for div in divs:
                q = div.find('a', class_="question_link")
                question_url = Zhihu_URL + q['href']
                question_title = q.text
                question = Question(question_url, question_title,
                                    session=self._session)

                ans = div.find('a', class_='answer-date-link')
                answer_url = Zhihu_URL + ans['href']

                up = div.find('a', class_='zm-item-vote-count')
                upvote = int(up['data-votecount'])

                au = div.find('div', class_='zm-item-answer-author-info')
                if au.a is None:
                    author_url = None
                    author_name = '匿名用户'
                    author_motto = ''
                else:
                    author_url = Zhihu_URL + au.a['href']
                    author_name = au.a.text
                    author_motto = au.strong['title'] if au.strong else ''
                author = Author(author_url, author_name, author_motto,
                                session=self._session)
                yield Answer(answer_url, question, author, upvote,
                             session=self._session)

            params['offset'] = last_score
            res = self._session.post(newest_url, data=params)
            gotten_feed_num = res.json()['msg'][0]
            # 如果得到内容数量为0则返回
            if gotten_feed_num == 0:
                return
            soup = BeautifulSoup(res.json()['msg'][1])

    @property
    def hot_questions(self):
        """获取话题下热门的问题

        :return: 话题下的热门动态中的问题，按热门度顺序返回生成器
        :rtype: Question.Iterable
        """
        from .question import Question
        hot_questions_url = Topic_Hot_Questions_Url.format(self.id)
        params = {'start': 0, '_xsrf': self.xsrf}
        res = self._session.get(hot_questions_url)
        soup = BeautifulSoup(res.content)
        while True:
            questions_duplicate = soup.find_all('a', class_='question_link')
            # 如果话题下无问题，则直接返回
            if len(questions_duplicate) == 0:
                return
                # 去除重复的问题
            questions = list(set(questions_duplicate))
            questions.sort(key=self._get_score, reverse=True)
            last_score = soup.find_all(
                'div', class_='feed-item')[-1]['data-score']
            for q in questions:
                question_url = Zhihu_URL + q['href']
                question_title = q.text
                question = Question(question_url, question_title,
                                    session=self._session)
                yield question
            params['offset'] = last_score
            res = self._session.post(hot_questions_url, data=params)
            gotten_feed_num = res.json()['msg'][0]
            # 如果得到问题数量为0则返回
            if gotten_feed_num == 0:
                return
            soup = BeautifulSoup(res.json()['msg'][1])

    @property
    def hot_answers(self):
        """获取话题下热门的回答

        :return: 话题下的热门动态中的回答，按热门度顺序返回生成器
        :rtype: Question.Iterable
        """
        from .question import Question
        from .author import Author
        from .answer import Answer
        hot_questions_url = Topic_Hot_Questions_Url.format(self.id)
        params = {'start': 0, '_xsrf': self.xsrf}
        res = self._session.get(hot_questions_url)
        soup = BeautifulSoup(res.content)
        while True:
            answers_div = soup.find_all('div', class_='feed-item')
            last_score = answers_div[-1]['data-score']
            for div in answers_div:
                # 没有 text area 的情况是：答案被和谐。
                if not (div.h3 and div.textarea):
                    continue
                question_url = Zhihu_URL + div.h2.a['href']
                question_title = div.a.text
                question = Question(question_url, question_title,
                                    session=self._session)
                if div.h3.a is None:
                    author_url = None
                    author_name = '匿名用户'
                    author_motto = ''
                else:
                    author_url = Zhihu_URL + div.h3.a['href']
                    author_name = div.h3.a.text
                    author_motto = div.strong['title'] if div.strong else ''
                author = Author(author_url, author_name, author_motto,
                                session=self._session)

                answer_url = Zhihu_URL + BeautifulSoup(
                    ''.join(map(str, div.textarea.contents))).find(
                    'a', class_='answer-date-link')['href']
                upvote_num = int(div.find(
                    'a', class_='zm-item-vote-count')['data-votecount'])

                yield Answer(answer_url, question, author, upvote_num,
                             session=self._session)

            params['offset'] = last_score
            res = self._session.post(hot_questions_url, data=params)
            gotten_feed_num = res.json()['msg'][0]
            # 如果得到问题数量为0则返回
            if gotten_feed_num == 0:
                return
            soup = BeautifulSoup(res.json()['msg'][1])

    @staticmethod
    def _get_score(tag):
        h2 = tag.parent
        div = h2.parent
        try:
            _ = h2['class']
            return div['data-score']
        except KeyError:
            return div.parent.parent['data-score']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import datetime

from .common import *
from .acttype import ActType
from .question import Question
from .answer import Answer
from .column import Column
from .post import Post
from .topic import Topic
from .author import Author


class Activity:

    """用户动态类，请使用Author.activities获取."""
    def __init__(self, act, session):
        """创建用户动态类实例.

        :param <class 'bs4.element.Tag'> act: 表示用户动态的页面元素
        :param Session session 使用的网络会话
        :return: 用户动态对象
        :rtype: Activity

        :说明:
            根据Activity.type的不同可以获取的属性，具体请看 :class:`.ActType`

        """
        self._session = session
        self.type = ActType.from_str(act.attrs['data-type-detail'])

        useless_tag = act.div.find('a', class_='zg-link')
        if useless_tag is not None:
            useless_tag.extract()

        attribute = self._get_assemble_method(self.type)(act)
        self._attr = attribute.__class__.__name__.lower()
        setattr(self, self._attr, attribute)
        self._time = datetime.fromtimestamp(int(act['data-time']))

    @property
    def content(self):
        """获取此对象中能提供的那个属性，对应表请查看ActType类.

        :return: 对象提供的对象
        :rtype: Author or Question or Answer or Topic or Column or Post
        """
        return getattr(self, self._attr)

    @property
    def time(self):
        """
        :return: 返回用户执行 Activity 操作的时间
        :rtype: datetime.datetime
        """
        return self._time

    def __find_post(self, act):
        column_url = act.find('a', class_='column_link')['href']
        column_name = act.find('a', class_='column_link').text
        column = Column(column_url, column_name, session=self._session)
        try:
            author_tag = act.find('div', class_='author-info')
            author_url = Zhihu_URL + author_tag.a['href']
            author_info = list(author_tag.stripped_strings)
            author_name = author_info[0]
            author_motto = author_info[1] \
                if len(author_info) > 1 else ''
        except TypeError:
            author_url = None
            author_name = '匿名用户'
            author_motto = ''
        author = Author(author_url, author_name, author_motto, session=self._session)
        post_url = act.find('a', class_='post-link')['href']
        post_title = act.find('a', class_='post-link').text
        post_comment_num, post_upvote_num = self._parse_un_cn(act)
        return Post(post_url, column, author, post_title,
                    post_upvote_num, post_comment_num,
                    session=self._session)

    def _assemble_create_post(self, act):
        return self.__find_post(act)

    def _assemble_voteup_post(self, act):
        return self.__find_post(act)

    def _assemble_follow_column(self, act):
        return Column(act.div.a['href'], act.div.a.text, session=self._session)

    def _assemble_follow_topic(self, act):
        topic_url = Zhihu_URL + act.div.a['href']
        topic_name = act.div.a['title']
        return Topic(topic_url, topic_name, session=self._session)

    def _assemble_answer_question(self, act):
        question_url = Zhihu_URL + re_a2q.match(act.div.find_all('a')[-1]['href']).group(1)
        question_title = act.div.find_all('a')[-1].text
        question = Question(question_url, question_title, session=self._session)
        answer_url = Zhihu_URL + act.div.find_all('a')[-1]['href']
        answer_comment_num, answer_upvote_num = self._parse_un_cn(act)
        return Answer(answer_url, question, self, answer_upvote_num, session=self._session)

    def _assemble_voteup_answer(self, act):
        question_url = Zhihu_URL + re_a2q.match(act.div.a['href']).group(1)
        question_title = act.div.a.text
        question = Question(question_url, question_title, session=self._session)
        try_find_author = act.find_all('a', class_='author-link', href=re.compile('^/people/[^/]*$'))

        if len(try_find_author) == 0:
            author_url = None
            author_name = '匿名用户'
            author_motto = ''
            photo_url = None
        else:
            try_find_author = try_find_author[-1]
            author_url = Zhihu_URL + try_find_author['href']
            author_name = try_find_author.text
            try_find_motto = try_find_author.parent.span
            if try_find_motto is None:
                author_motto = ''
            else:
                author_motto = try_find_motto['title']
            photo_url = PROTOCOL + try_find_author.parent.a.img[
                'src'].replace('_s', '_r')

        author = Author(author_url, author_name, author_motto,
                        photo_url=photo_url, session=self._session)
        answer_url = Zhihu_URL + act.div.a['href']
        answer_comment_num, answer_upvote_num = self._parse_un_cn(act)
        return Answer(answer_url, question, author, answer_upvote_num, session=self._session)

    def _assemble_ask_question(self, act):
        return Question(Zhihu_URL + act.div.contents[3]['href'],
                        list(act.div.children)[3].text,
                        session=self._session)

    def _assemble_follow_question(self, act):
        return Question(Zhihu_URL + act.div.a['href'], act.div.a.text, session=self._session)

    def _get_assemble_method(self, act_type):
        assemble_methods = {
            ActType.UPVOTE_POST: self._assemble_voteup_post,
            ActType.FOLLOW_COLUMN: self._assemble_follow_column,
            ActType.UPVOTE_ANSWER: self._assemble_voteup_answer,
            ActType.ANSWER_QUESTION: self._assemble_answer_question,
            ActType.ASK_QUESTION: self._assemble_ask_question,
            ActType.FOLLOW_QUESTION: self._assemble_follow_question,
            ActType.FOLLOW_TOPIC: self._assemble_follow_topic,
            ActType.PUBLISH_POST: self._assemble_create_post,
        }

        if act_type in assemble_methods:
            return assemble_methods[act_type]
        else:
            raise ValueError('invalid activity type')

    @staticmethod
    def _parse_un_cn(act):
        upvote_num = int(act.find('a', class_='zm-item-vote-count')['data-votecount'])
        comment = act.find('a', class_='toggle-comment')
        comment_text = next(comment.stripped_strings)
        comment_num_match = re_get_number.match(comment_text)
        comment_num = int(comment_num_match.group(1)) if comment_num_match is not None else 0
        return comment_num, upvote_num



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import time
import json
import requests
import importlib

from .common import *


class ZhihuClient:

    """知乎客户端类，内部维护了自己专用的网络会话，可用cookies或账号密码登录."""

    def __init__(self, cookies=None):
        """创建客户端类实例.

        :param str cookies: 见 :meth:`.login_with_cookies` 中 ``cookies`` 参数
        :return: 知乎客户端对象
        :rtype: ZhihuClient
        """
        self._session = requests.Session()
        self._session.headers.update(Default_Header)
        if cookies is not None:
            assert isinstance(cookies, str)
            self.login_with_cookies(cookies)

    # ===== login staff =====

    @staticmethod
    def _get_captcha_url():
        return Captcha_URL_Prefix + str(int(time.time() * 1000))

    def get_captcha(self):
        """获取验证码数据。

        :return: 验证码图片数据。
        :rtype: bytes
        """
        # some unbelievable zhihu logic
        self._session.get(Zhihu_URL)
        data = {'email': '', 'password': '', 'remember_me': 'true'}
        self._session.post(Login_URL, data=data)

        r = self._session.get(self._get_captcha_url())
        return r.content

    def login(self, email, password, captcha):
        """登陆知乎.

        :param str email: 邮箱
        :param str password: 密码
        :param str captcha: 验证码
        :return:
            ======== ======== ============== ====================
            元素序号 元素类型 意义           说明
            ======== ======== ============== ====================
            0        int      是否成功       0为成功，1为失败
            1        str      失败原因       登录成功则为空字符串
            2        str       cookies字符串 登录失败则为空字符串
            ======== ======== ============== ====================

        :rtype: (int, str, str)
        """
        data = {'email': email, 'password': password,
                'remember_me': 'true', 'captcha': captcha}
        r = self._session.post(Login_URL, data=data)
        j = r.json()
        code = int(j['r'])
        message = j['msg']
        cookies_str = json.dumps(self._session.cookies.get_dict()) \
            if code == 0 else ''
        return code, message, cookies_str

    def login_with_cookies(self, cookies):
        """使用cookies文件或字符串登录知乎

        :param str cookies:
            ============== ===========================
            参数形式       作用
            ============== ===========================
            文件名         将文件内容作为cookies字符串
            cookies字符串  直接提供cookies字符串
            ============== ===========================
        :return: 无
        :rtype: None
        """
        if os.path.isfile(cookies):
            with open(cookies) as f:
                cookies = f.read()
        cookies_dict = json.loads(cookies)
        self._session.cookies.update(cookies_dict)

    def login_in_terminal(self):
        """不使用cookies，在终端中根据提示登陆知乎

        :return: 如果成功返回cookies字符串
        :rtype: str
        """
        print('====== zhihu login =====')

        email = input('email: ')
        password = input('password: ')

        captcha_data = self.get_captcha()
        with open('captcha.gif', 'wb') as f:
            f.write(captcha_data)

        print('please check captcha.gif for captcha')
        captcha = input('captcha: ')
        os.remove('captcha.gif')

        print('====== logging.... =====')

        code, msg, cookies = self.login(email, password, captcha)

        if code == 0:
            print('login successfully')
        else:
            print('login failed, reason: {0}'.format(msg))

        return cookies

    def create_cookies(self, file):
        cookies_str = self.login_in_terminal()
        if cookies_str:
            with open(file, 'w') as f:
                f.write(cookies_str)
            print('cookies file created.')
        else:
            print('can\'t create cookies.')

    # ===== network staff =====

    def set_proxy(self, proxy):
        """设置代理

        :param str proxy: 使用 "http://example.com:port" 的形式
        :return: 无
        :rtype: None

        :说明:
            由于一个 :class:`.ZhihuClient` 对象和它创建出来的其他知乎对象共用
            一个Session，所以调用这个方法也会将所有生成出的知乎类设置上代理。
        """
        self._session.proxies.update({'http': proxy})

    # ===== getter staff ======

    def me(self):
        """获取使用特定cookies的Me实例

        :return: cookies对应的Me对象
        :rtype: Me
        """
        from .me import Me
        headers = dict(Default_Header)
        headers['Host'] = 'zhuanlan.zhihu.com'
        res = self._session.get(Get_Me_Info_Url, headers=headers)
        json_data = res.json()
        url = json_data['profileUrl']
        name = json_data['name']
        motto = json_data['bio']
        photo = json_data['avatar']['template'].format(
            id=json_data['avatar']['id'], size='r')
        return Me(url, name, motto, photo, session=self._session)

    def __getattr__(self, item: str):
        """本函数用于获取各种类，如 `Answer` `Question` 等.

        :支持的形式有:
            1. client.answer()
            2. client.author()
            3. client.collection()
            4. client.column()
            5. client.post()
            6. client.question()
            7. client.topic()

            参数均为对应页面的url，返回对应的类的实例。
        """
        def getter(url):
            return getattr(module, item.capitalize())(url,
                                                      session=self._session)
        attr_list = ['answer', 'author', 'collection',
                     'column', 'post', 'question', 'topic']
        if item.lower() in attr_list:
            module = importlib.import_module('.'+item.lower(), 'zhihu')
            return getter



from .common import BeautifulSoup
from requests import Response
import json


class BaseZhihu:
    def _gen_soup(self, content):
        self.soup = BeautifulSoup(content)

    def _get_content(self):
        return self._session.get(self.url).content

    def _make_soup(self):
        if self.url and not self.soup:
            content = self._get_content()
            self._gen_soup(content)

    @classmethod
    def from_html(cls, content):
        obj = cls(url=None)
        obj._gen_soup(content)
        return obj


class JsonAsSoupMixin:
    def _gen_soup(self, content):
        # 为了让`from_html`对外提供统一的接口, 判断一下输入, 如果是bytes 或者 str 则用json处理,
        # 否则认为是由_get_content返回的dict

        if isinstance(content, bytes):
            r = Response()
            r._content = content
            soup = r.json()
            self.soup = soup
        elif isinstance(content, str):
            self.soup = json.loads(content)
        else:
            self.soup = content



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os

from test_utils import TEST_DATA_PATH
from zhihu import Collection
from zhihu.common import BeautifulSoup


class CollectionTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://www.zhihu.com/collection/28698204'
        file_path = os.path.join(TEST_DATA_PATH, 'collection.html')
        with open(file_path, 'rb') as f:
            html = f.read()
        soup = BeautifulSoup(html)

        cls.collection = Collection(url)
        cls.collection._session = None
        cls.collection.soup = soup
        cls.expected = {'cid': 3725428, 'name': '可以用来背的答案',
                        'xsrf': 'cfd489623d34ca03adfdc125368c6426',
                        'owner_id': 'buhuilengyoumo', 'owner_name': '树叶',
                        'follower_num': 6328, 'top_ques_id': 26092705,
                        'top_ques_title': ('一直追求（吸引）不到喜欢的异性，'
                                           '感觉累了怎么办？'),
                        'top_ans_id': 32989919, 'top_ans_author_name': '朱炫',
                        'top_ans_upvote_num': 16595,
                        'top_ans_author_id': 'zhu-xuan-86'
                        }

    def test_cid(self):
        self.assertEqual(self.expected['cid'], self.collection.cid)

    def test_name(self):
        self.assertEqual(self.expected['name'], self.collection.name)

    def test_xsrf(self):
        self.assertEqual(self.expected['xsrf'], self.collection.xsrf)

    def test_owner(self):
        owner = self.collection.owner
        self.assertEqual(self.expected['owner_id'], owner.id)
        self.assertEqual(self.expected['owner_name'], owner.name)

    def test_follower_num(self):
        self.assertEqual(self.expected['follower_num'],
                         self.collection.follower_num)

    def test_page_get_questions(self):
        questions = [q for q in
                     self.collection._page_get_questions(self.collection.soup)]
        ques = questions[0]
        self.assertEqual(self.expected['top_ques_id'], ques.id)
        self.assertEqual(self.expected['top_ques_title'], ques.title)

    def test_page_get_answers(self):
        answers = [a for a in
                   self.collection._page_get_answers(self.collection.soup)]
        ans = answers[0]
        self.assertEqual(self.expected['top_ans_id'], ans.id)
        self.assertEqual(self.expected['top_ans_upvote_num'], ans.upvote_num)
        self.assertEqual(self.expected['top_ans_author_name'], ans.author.name)
        self.assertEqual(self.expected['top_ans_author_id'], ans.author.id)

    def test_questions(self):
        qs = self.collection.questions
        ques = next(qs)
        self.assertEqual(self.expected['top_ques_id'], ques.id)
        self.assertEqual(self.expected['top_ques_title'], ques.title)

    def test_answers(self):
        anses = self.collection.answers
        ans = next(anses)
        self.assertEqual(self.expected['top_ans_id'], ans.id)
        self.assertEqual(self.expected['top_ans_upvote_num'], ans.upvote_num)
        self.assertEqual(self.expected['top_ans_author_name'], ans.author.name)
        self.assertEqual(self.expected['top_ans_author_id'], ans.author.id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json

from .common import *
from .author import Author


class Me(Author):
    """封装了相关操作（如点赞，关注问题）的类。
    请使用 :meth:`.ZhihuClient.me` 方法获取实例。
    """

    def __init__(self, url, name, motto, photo_url, session):
        super(Me, self).__init__(url, name, motto,
                                 photo_url=photo_url, session=session)

    def vote(self, something, vote='up'):
        """给答案或文章点赞或取消点赞

        :param Answer/Post something: 需要点赞的答案或文章对象
        :param str vote:
            ===== ================ ======
            取值        说明       默认值
            ===== ================ ======
            up    赞同              √
            down  反对              X
            clear 既不赞同也不反对  X
            ===== ================ ======

        :return: 成功返回True，失败返回False
        :rtype: bool
        """
        from .answer import Answer
        from zhihu import Post
        if isinstance(something, Answer):
            mapping = {
                'up': 'vote_up',
                'clear': 'vote_neutral',
                'down': 'vote_down'
            }
            if vote not in mapping.keys():
                raise ValueError('Invalid vote value: {0}'.format(vote))
            if something.author.url == self.url:
                return False
            params = {'answer_id': str(something.aid)}
            data = {
                '_xsrf': something.xsrf,
                'method': mapping[vote],
                'params': json.dumps(params)
            }
            headers = dict(Default_Header)
            headers['Referer'] = something.question.url[:-1]
            res = self._session.post(Upvote_Answer_Url,
                                     headers=headers, data=data)
            return res.json()['r'] == 0
        elif isinstance(something, Post):
            mapping = {
                'up': 'like',
                'clear': 'none',
                'down': 'dislike'
            }
            if vote not in mapping.keys():
                raise ValueError('Invalid vote value: {0}'.format(vote))
            if something.author.url == self.url:
                return False
            put_url = Upvote_Article_Url.format(
                something.column_in_name, something.slug)
            data = {'value': mapping[vote]}
            headers = {
                'Content-Type': 'application/json;charset=utf-8',
                'Host': 'zhuanlan.zhihu.com',
                'Referer': something.url[:-1],
                'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; '
                              'rv:39.0) Gecko/20100101 Firefox/39.0',
                'X-XSRF-TOKEN': self._session.cookies.get('XSRF-TOKEN')
            }
            res = self._session.put(put_url, json.dumps(data), headers=headers)
            return res.status_code == 204
        else:
            raise ValueError('argument something need to be '
                             'zhihu.Answer or zhihu.Post object.')

    def thanks(self, answer, thanks=True):
        """感谢或取消感谢回答

        :param Answer answer: 要感谢或取消感谢的回答
        :param thanks: True-->感谢，False-->取消感谢
        :return: 成功返回True，失败返回False
        :rtype: bool
        """
        from .answer import Answer
        if isinstance(answer, Answer) is False:
            raise ValueError('argument answer need to be Zhihu.Answer object.')
        if answer.author.url == self.url:
                return False
        data = {
            '_xsrf': answer.xsrf,
            'aid': answer.aid
        }
        res = self._session.post(Thanks_Url if thanks else Cancel_Thanks_Url,
                                 data=data)
        return res.json()['r'] == 0

    def follow(self, something, follow=True):
        """关注用户、问题、话题或收藏夹

        :param Author/Question/Topic something: 需要关注的对象
        :param bool follow: True-->关注，False-->取消关注
        :return: 成功返回True，失败返回False
        :rtype: bool
        """
        from .question import Question
        from .topic import Topic
        from .collection import Collection
        if isinstance(something, Author):
            if something.url == self.url:
                return False
            data = {
                '_xsrf': something.xsrf,
                'method': '	follow_member' if follow else 'unfollow_member',
                'params': json.dumps({'hash_id': something.hash_id})
            }
            res = self._session.post(Follow_Author_Url, data=data)
            return res.json()['r'] == 0
        elif isinstance(something, Question):
            data = {
                '_xsrf': something.xsrf,
                'method': 'follow_question' if follow else 'unfollow_question',
                'params': json.dumps({'question_id': str(something.qid)})
            }
            res = self._session.post(Follow_Question_Url, data=data)
            return res.json()['r'] == 0
        elif isinstance(something, Topic):
            data = {
                '_xsrf': something.xsrf,
                'method': 'follow_topic' if follow else 'unfollow_topic',
                'params': json.dumps({'topic_id': something.tid})
            }
            res = self._session.post(Follow_Topic_Url, data=data)
            return res.json()['r'] == 0
        elif isinstance(something, Collection):
            data = {
                '_xsrf': something.xsrf,
                'favlist_id': something.cid
            }
            res = self._session.post(
                Follow_Collection_Url if follow else Unfollow_Collection_Url,
                data=data)
            return res.json()['r'] == 0
        else:
            raise ValueError('argument something need to be '
                             'zhihu.Author, zhihu.Question'
                             ', Zhihu.Topic or Zhihu.Collection object.')

    def add_comment(self, answer, content):
        """给指定答案添加评论

        :param Answer answer: 答案对象
        :param string content: 评论内容
        :return: 成功返回 True，失败返回 False
        :rtype: bool
        """

        from .answer import Answer
        if isinstance(answer, Answer) is False:
            raise ValueError('argument answer need to be Zhihu.Answer object.')
        if not content:
            raise ValueError('answer content cannot be empty')
        data = {
            'method': 'add_comment',
            'params': json.dumps({'answer_id': answer.aid, 'content': content}),
            '_xsrf': answer.xsrf
            }
        res = self._session.post(Answer_Add_Comment_URL,
                                 data=data)
        return res.json()['r'] == 0

    def send_message(self, author, content):
        """发送私信给一个用户

        :param Author author: 接收私信用户对象
        :param string content: 发送给用户的私信内容
        :return: 成功返回 True，失败返回 False
        :rtype: bool
        """
        if isinstance(author, Author) is False:
            raise ValueError('argument answer need to be Zhihu.Author object.')
        if not content:
            raise ValueError('answer content cannot be empty')
        if author.url == self.url:
                return False
        data = {
            'member_id': author.hash_id,
            'content': content,
            'token': '',
            '_xsrf': author.xsrf
        }
        res = self._session.post(Send_Message_Url,
                                 data=data)
        return res.json()['r'] == 0

    def block(self, something, block=True):
        """屏蔽某个用户、话题

        :param Author/Topic something:
        :param block: True-->屏蔽，False-->取消屏蔽
        :return: 成功返回 True，失败返回 False
        :rtype: bool
        """
        from .topic import Topic

        if isinstance(something, Author):

            if something.url == self.url:
                return False
            data = {
                '_xsrf': something.xsrf,
                'action': 'add' if block else 'cancel',
            }
            block_author_url = something.url + 'block'
            res = self._session.post(block_author_url, data=data)
            return res.json()['r'] == 0
        elif isinstance(something, Topic):
            tid = something.tid
            data = {
                '_xsrf': something.xsrf,
                'method': 'add' if block else 'del',
                'tid': tid,
            }
            block_topic_url = 'http://www.zhihu.com/topic/ignore'
            res = self._session.post(block_topic_url, data=data)
            return res.status_code == 200
        else:
            raise ValueError('argument something need to be '
                             'Zhihu.Author or Zhihu.Topic object.')

    def unhelpful(self, answer, unhelpful=True):
        """没有帮助或取消没有帮助回答

        :param Answer answer: 要没有帮助或取消没有帮助回答
        :param unhelpful: True-->没有帮助，False-->取消没有帮助
        :return: 成功返回 True，失败返回 False
        :rtype: bool
        """
        from .answer import Answer
        if isinstance(answer, Answer) is False:
            raise ValueError('argument answer need to be Zhihu.Answer object.')
        if answer.author.url == self.url:
                return False
        data = {
            '_xsrf': answer.xsrf,
            'aid': answer.aid
        }
        res = self._session.post(Unhelpful_Url if unhelpful else Cancel_Unhelpful_Url,
                                 data=data)
        return res.json()['r'] == 0



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json

from .common import *
from .base import BaseZhihu


class Author(BaseZhihu):

    """用户类，请使用``ZhihuClient.answer``方法构造对象."""

    @class_common_init(re_author_url, True)
    def __init__(self, url, name=None, motto=None, follower_num=None,
                 question_num=None, answer_num=None, upvote_num=None,
                 thank_num=None, photo_url=None, session=None):
        """创建用户类实例.

        :param str url: 用户主页url，形如 http://www.zhihu.com/people/7sdream
        :param str name: 用户名字，可选
        :param str motto: 用户简介，可选
        :param int follower_num: 用户粉丝数，可选
        :param int question_num: 用户提问数，可选
        :param int answer_num: 用户答案数，可选
        :param int upvote_num: 用户获得赞同数，可选
        :param int thank_num: 用户获得感谢数，可选
        :param str photo_url: 用户头像地址，可选
        :param Session session: 使用的网络会话，为空则使用新会话。
        :return: 用户对象
        :rtype: Author
        """
        self.url = url
        self._session = session
        self.card = None
        self._nav_list = None
        self._name = name
        self._motto = motto
        self._follower_num = follower_num
        self._question_num = question_num
        self._answer_num = answer_num
        self._upvote_num = upvote_num
        self._thank_num = thank_num
        self._photo_url = photo_url

    def _gen_soup(self, content):
        self.soup = BeautifulSoup(content)
        self._nav_list = self.soup.find(
            'div', class_='profile-navbar').find_all('a')

    def _make_card(self):
        if self.card is None and self.url is not None:
            params = {'url_token': self.id}
            real_params = {'params': json.dumps(params)}
            r = self._session.get(Get_Profile_Card_URL, params=real_params)
            self.card = BeautifulSoup(r.content)

    @property
    def id(self):
        """获取用户id，就是网址最后那一部分.

        :return: 用户id
        :rtype: str
        """
        return re.match(r'^.*/([^/]+)/$', self.url).group(1) if self.url is not None else ''

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        self._make_soup()
        return self.soup.find(
            'input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_hash_id')
    def hash_id(self):
        """获取作者的内部hash id（用不到就忽视吧~）

        :return: 用户hash id
        :rtype: str
        """
        div = self.soup.find('div', class_='zm-profile-header-op-btns')
        if div is not None:
            return div.button['data-id']
        else:
            ga = self.soup.find('script', attrs={'data-name': 'ga_vars'})
            return json.loads(ga.text)['user_hash']

    @property
    @check_soup('_name', '_make_card')
    def name(self):
        """获取用户名字.

        :return: 用户名字
        :rtype: str
        """
        if self.url is None:
            return '匿名用户'
        if self.soup is not None:
            return self.soup.find('div', class_='title-section').span.text
        else:
            assert self.card is not None
            return self.card.find('span', class_='name').text

    @property
    @check_soup('_motto', '_make_card')
    def motto(self):
        """获取用户自我介绍，由于历史原因，我还是把这个属性叫做motto吧.

        :return: 用户自我介绍
        :rtype: str
        """
        if self.url is None:
            return ''
        else:
            if self.soup is not None:
                bar = self.soup.find(
                    'div', class_='title-section')
                if len(bar.contents) < 4:
                    return ''
                else:
                    return bar.contents[3].text
            else:
                assert self.card is not None
                motto = self.card.find('div', class_='tagline')
                return motto.text if motto is not None else ''

    @property
    @check_soup('_photo_url', '_make_card')
    def photo_url(self):
        """获取用户头像图片地址.

        :return: 用户头像url
        :rtype: str
        """
        if self.url is not None:
            if self.soup is not None:
                img = 'http:' + self.soup.find(
                    'img', class_='avatar avatar-l')['src']
                return img.replace('_l', '_r')
            else:
                assert(self.card is not None)
                return PROTOCOL + self.card.img['src'].replace('_xs', '_r')
        else:
            return 'http://pic1.zhimg.com/da8e974dc_r.jpg'

    @property
    @check_soup('_followee_num')
    def followee_num(self):
        """获取关注了多少人.

        :return: 关注的人数
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'div', class_='zm-profile-side-following').a.strong.text)
            return number

    @property
    @check_soup('_follower_num')
    def follower_num(self):
        """获取追随者数量，就是关注此人的人数.

        :return: 追随者数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'div', class_='zm-profile-side-following zg-clear').find_all(
                'a')[1].strong.text)
            return number

    @property
    @check_soup('_upvote_num')
    def upvote_num(self):
        """获取收到的的赞同数量.

        :return: 收到的的赞同数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'span', class_='zm-profile-header-user-agree').strong.text)
            return number

    @property
    @check_soup('_thank_num')
    def thank_num(self):
        """获取收到的感谢数量.

        :return: 收到的感谢数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            number = int(self.soup.find(
                'span', class_='zm-profile-header-user-thanks').strong.text)
            return number

    @property
    @check_soup('_weibo_url')
    def weibo_url(self):
        """获取用户微博链接.

        :return: 微博链接地址，如没有则返回 ‘unknown’
        :rtype: str
        """
        if self.url is None:
            return None
        else:
            tmp = self.soup.find(
                'a', class_='zm-profile-header-user-weibo')
            return tmp['href'] if tmp is not None else 'unknown'

    @property
    def business(self):
        """用户的行业.

        :return: 用户的行业，如没有则返回 ‘unknown’
        :rtype: str
        """
        return self._find_user_profile('business')

    @property
    def location(self):
        """用户的所在地.

        :return: 用户的所在地，如没有则返回 ‘unknown’
        :rtype: str
        """
        return self._find_user_profile('location')

    @property
    def education(self):
        """用户的教育状况.

        :return: 用户的教育状况，如没有则返回 ‘unknown’
        :rtype: str
        """
        return self._find_user_profile('education')

    def _find_user_profile(self, t):
        self._make_soup()
        if self.url is None:
            return 'unknown'
        else:
            res = self.soup.find(
                'span', class_=t)
            if res:
                return res['title']
            else:
                return 'unknown'

    @property
    @check_soup('_gender')
    def gender(self):
        """用户的性别.

        :return: 用户的性别（male/female/unknown)
        :rtype: str
        """
        if self.url is None:
            return 'unknown'
        else:
            return 'female' if self.soup.find('i', class_='icon-profile-female') else 'male'

    @property
    @check_soup('_question_num')
    def question_num(self):
        """获取提问数量.

        :return: 提问数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[1].span.text)

    @property
    @check_soup('_answer_num')
    def answer_num(self):
        """获取答案数量.

        :return: 答案数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[2].span.text)

    @property
    @check_soup('_post_num')
    def post_num(self):
        """获取专栏文章数量.

        :return: 专栏文章数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[3].span.text)

    @property
    @check_soup('_collection_num')
    def collection_num(self):
        """获取收藏夹数量.

        :return: 收藏夹数量
        :rtype: int
        """
        if self.url is None:
            return 0
        else:
            return int(self._nav_list[4].span.text)

    @property
    @check_soup('_followed_column_num')
    def followed_column_num(self):
        """获取用户关注的专栏数

        :return: 关注的专栏数
        :rtype: int
        """
        if self.url is not None:
            tag = self.soup.find('div', class_='zm-profile-side-columns')
            if tag is not None:
                return int(re_get_number.match(tag.parent.strong.text).group(1))
        return 0

    @property
    @check_soup('_followed_topic_num')
    def followed_topic_num(self):
        """获取用户关注的话题数

        :return: 关注的话题数
        :rtype: int
        """
        if self.url is not None:
            tag = self.soup.find('div', class_='zm-profile-side-topics')
            if tag is not None:
                return int(re_get_number.match(tag.parent.strong.text).group(1))
        return 0

    @property
    def questions(self):
        """获取用户的所有问题.

        :return: 用户的所有问题，返回生成器.
        :rtype: Question.Iterable
        """
        from .question import Question

        if self.url is None or self.question_num == 0:
            return
        for page_index in range(1, (self.question_num - 1) // 20 + 2):
            html = self._session.get(
                self.url + 'asks?page=' + str(page_index)).text
            soup = BeautifulSoup(html)
            question_links = soup.find_all('a', class_='question_link')
            question_datas = soup.find_all(
                'div', class_='zm-profile-section-main')
            for link, data in zip(question_links, question_datas):
                url = Zhihu_URL + link['href']
                title = link.text
                answer_num = int(
                    re_get_number.match(data.div.contents[4]).group(1))
                follower_num = int(
                    re_get_number.match(data.div.contents[6]).group(1))
                q = Question(url, title, follower_num, answer_num,
                             session=self._session)
                yield q

    @property
    def answers(self):
        """获取用户的所有答案.

        :return: 用户所有答案，返回生成器.
        :rtype: Answer.Iterable
        """
        from .question import Question
        from .answer import Answer

        if self.url is None or self.answer_num == 0:
            return
        for page_index in range(1, (self.answer_num - 1) // 20 + 2):
            html = self._session.get(
                self.url + 'answers?page=' + str(page_index)).text
            soup = BeautifulSoup(html)
            questions = soup.find_all('a', class_='question_link')
            upvotes = soup.find_all('a', class_='zm-item-vote-count')
            for q, upvote in zip(questions, upvotes):
                answer_url = Zhihu_URL + q['href']
                question_url = Zhihu_URL + re_a2q.match(q['href']).group(1)
                question_title = q.text
                upvote = int(upvote['data-votecount'])
                question = Question(question_url, question_title,
                                    session=self._session)
                yield Answer(answer_url, question, self, upvote,
                             session=self._session)

    @property
    def followers(self):
        """获取关注此用户的人.

        :return: 关注此用户的人，返回生成器
        :rtype: Author.Iterable
        """
        for x in self._follow_ee_ers('er'):
            yield x

    @property
    def followees(self):
        """获取用户关注的人.

        :return: 用户关注的人的，返回生成器
        :rtype: Author.Iterable
        """
        for x in self._follow_ee_ers('ee'):
            yield x

    def _follow_ee_ers(self, t):
        if self.url is None:
            return
        if t == 'er':
            request_url = Author_Get_More_Followers_URL
        else:
            request_url = Author_Get_More_Followees_URL
        self._make_card()
        if self.hash_id is None:
            self._make_soup()
        headers = dict(Default_Header)
        headers['Referer'] = self.url + 'follow' + t + 's'
        params = {"order_by": "created", "offset": 0, "hash_id": self.hash_id}
        data = {'_xsrf': self.xsrf, 'method': 'next', 'params': ''}
        gotten_date_num = 20
        offset = 0
        while gotten_date_num == 20:
            params['offset'] = offset
            data['params'] = json.dumps(params)
            res = self._session.post(request_url, data=data, headers=headers)
            json_data = res.json()
            gotten_date_num = len(json_data['msg'])
            offset += gotten_date_num
            for html in json_data['msg']:
                soup = BeautifulSoup(html)
                h2 = soup.find('h2')
                author_name = h2.a.text
                author_url = h2.a['href']
                author_motto = soup.find('div', class_='zg-big-gray').text
                author_photo = PROTOCOL + soup.a.img['src'].replace('_m', '_r')
                numbers = [int(re_get_number.match(x.text).group(1))
                           for x in soup.find_all('a', target='_blank')]
                yield Author(author_url, author_name, author_motto, *numbers,
                             photo_url=author_photo, session=self._session)

    @property
    def collections(self):
        """获取用户收藏夹.

        :return: 用户收藏夹，返回生成器
        :rtype: Collection.Iterable
        """
        from .collection import Collection

        if self.url is None or self.collection_num == 0:
            return
        else:
            collection_num = self.collection_num
            for page_index in range(1, (collection_num - 1) // 20 + 2):
                html = self._session.get(
                    self.url + 'collections?page=' + str(page_index)).text
                soup = BeautifulSoup(html)
                collections_names = soup.find_all(
                    'a', class_='zm-profile-fav-item-title')
                collection_follower_nums = soup.find_all(
                    'div', class_='zm-profile-fav-bio')
                for c, f in zip(collections_names, collection_follower_nums):
                    c_url = Zhihu_URL + c['href']
                    c_name = c.text
                    c_fn = int(re_get_number.match(f.contents[2]).group(1))
                    yield Collection(c_url, self, c_name, c_fn,
                                     session=self._session)

    @property
    def columns(self):
        """获取用户专栏.

        :return: 用户专栏，返回生成器
        :rtype: Column.Iterable
        """
        from .column import Column

        if self.url is None or self.post_num == 0:
            return
        soup = BeautifulSoup(self._session.get(self.url + 'posts').text)
        column_tags = soup.find_all('div', class_='column')
        for column_tag in column_tags:
            name = column_tag.div.a.span.text
            url = column_tag.div.a['href']
            follower_num = int(re_get_number.match(
                column_tag.div.div.a.text).group(1))
            footer = column_tag.find('div', class_='footer')
            if footer is None:
                post_num = 0
            else:
                post_num = int(
                    re_get_number.match(footer.a.text).group(1))
            yield Column(url, name, follower_num, post_num,
                         session=self._session)

    @property
    def followed_columns(self):
        """获取用户关注的专栏.

        :return: 用户关注的专栏，返回生成器
        :rtype: Column.Iterable
        """
        from .column import Column
        if self.url is None:
            return
        if self.followed_column_num > 0:
            tag = self.soup.find('div', class_='zm-profile-side-columns')
            if tag is not None:
                for a in tag.find_all('a'):
                    yield Column(a['href'], a.img['alt'], session=self._session)
            if self.followed_column_num > 7:
                offset = 7
                gotten_data_num = 20
                while gotten_data_num == 20:
                    params = {'hash_id': self.hash_id, 'limit': 20, 'offset': offset}
                    data = {'method': 'next', '_xsrf': self.xsrf, 'params': json.dumps(params)}
                    j = self._session.post(Author_Get_More_Follow_Column_URL, data=data).json()
                    gotten_data_num = len(j['msg'])
                    offset += gotten_data_num
                    for msg in map(BeautifulSoup, j['msg']):
                        name = msg.strong.text
                        url = msg.a['href']
                        post_num = int(re_get_number.match(msg.span.text).group(1))
                        yield Column(url, name, post_num=post_num, session=self._session)

    @property
    def followed_topics(self):
        """获取用户关注的话题.

        :return: 用户关注的话题，返回生成器
        :rtype: Topic.Iterable
        """
        from .topic import Topic
        if self.url is None:
            return
        if self.followed_topic_num > 0:
            tag = self.soup.find('div', class_='zm-profile-side-topics')
            if tag is not None:
                for a in tag.find_all('a'):
                    yield Topic(Zhihu_URL + a['href'], a.img['title'], session=self._session)
            if self.followed_topic_num > 7:
                offset = 7
                gotten_data_num = 20
                while gotten_data_num == 20:
                    data = {'start': 0, 'offset': offset, '_xsrf': self.xsrf}
                    j = self._session.post(Author_Get_More_Follow_Topic_URL.format(self.id), data=data).json()
                    gotten_data_num = j['msg'][0]
                    offset += gotten_data_num
                    for div in BeautifulSoup(j['msg'][1]).find_all('div', class_='zm-profile-section-item'):
                        name = div.strong.text
                        url = Zhihu_URL + div.a['href']
                        yield Topic(url, name, session=self._session)

    @property
    def activities(self):
        """获取用户的最近动态.

        :return: 最近动态，返回生成器，具体说明见 :class:`.Activity`
        :rtype: Activity.Iterable
        """
        from .activity import Activity

        if self.url is None:
            return
        self._make_soup()
        gotten_feed_num = 20
        start = '0'
        while gotten_feed_num == 20:
            data = {'_xsrf': self.xsrf, 'start': start}
            res = self._session.post(self.url + 'activities', data=data)
            gotten_feed_num = res.json()['msg'][0]
            soup = BeautifulSoup(res.json()['msg'][1])
            acts = soup.find_all(
                'div', class_='zm-profile-section-item zm-item clearfix')

            start = acts[-1]['data-time'] if len(acts) > 0 else 0
            for act in acts:
                yield Activity(act, self._session)

    @property
    def last_activity_time(self):
        """获取用户最后一次活动的时间

        :return: 用户最后一次活动的时间，返回值为 unix 时间戳
        :rtype: int
        """
        self._make_soup()
        act = self.soup.find(
            'div', class_='zm-profile-section-item zm-item clearfix')
        return int(act['data-time']) if act is not None else -1

    def is_zero_user(self):
        """返回当前用户是否为三零用户，其实是四零： 赞同0，感谢0，提问0，回答0.

        :return: 是否是三零用户
        :rtype: bool
        """
        return self.upvote_num + self.thank_num + \
            self.question_num + self.answer_num == 0



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import json
from datetime import datetime

from .common import *
from .base import BaseZhihu


class Answer(BaseZhihu):
    """答案类，请使用``ZhihuClient.answer``方法构造对象."""

    @class_common_init(re_ans_url)
    def __init__(self, url, question=None, author=None,
                 upvote_num=None, content=None, session=None):
        """创建答案类实例.

        :param str url: 答案url
        :param Question question: 答案所在的问题对象，可选
        :param Author author: 答案回答者对象，可选
        :param int upvote_num: 答案赞同数量，可选
        :param str content: 答案内容，可选
        :param Session session: 使用的网络会话，为空则使用新会话
        :return: 答案对象
        :rtype: Answer
        """
        self.url = url
        self._session = session
        self._question = question
        self._author = author
        self._upvote_num = upvote_num
        self._content = content

    @property
    def id(self):
        """答案的id

        :return: 答案id
        :rtype: int
        """
        return int(re.match(r'.*/(\d+)/$', self.url).group(1))

    @property
    @check_soup('_xsrf')
    def xsrf(self):
        """获取知乎的反xsrf参数（用不到就忽视吧~）

        :return: xsrf参数
        :rtype: str
        """
        return self.soup.find(
            'input', attrs={'name': '_xsrf'})['value']

    @property
    @check_soup('_aid')
    def aid(self):
        """获取答案的内部id，某些POST操作需要此参数

        :return: 答案内部id
        :rtype: str
        """
        return int(self.soup.find(
            'div', class_='zm-item-answer')['data-aid'])

    @property
    @check_soup('_html')
    def html(self):
        """获取网页源码

        :return: 网页源码
        :rtype: str
        """
        return self.soup.prettify()

    @property
    @check_soup('_author')
    def author(self):
        """获取答案作者.

        :return: 答案作者
        :rtype: Author
        """
        from .author import Author

        author = self.soup.find('div', class_='zm-item-answer-author-info')
        url, name, motto, photo = parser_author_from_tag(author)
        return Author(url, name, motto, photo_url=photo, session=self._session)

    @property
    @check_soup('_question')
    def question(self):
        """获取答案所在问题.

        :return: 答案所在问题
        :rtype: Question
        """
        from .question import Question

        question_link = self.soup.find(
            "h2", class_="zm-item-title zm-editable-content").a
        url = Zhihu_URL + question_link["href"]
        title = question_link.text
        followers_num = int(self.soup.find(
            'div', class_='zh-question-followers-sidebar').div.a.strong.text)
        answers_num = int(re_get_number.match(self.soup.find(
            'div', class_='zh-answers-title').h3.a.text).group(1))
        return Question(url, title, followers_num, answers_num,
                        session=self._session)

    @property
    @check_soup('_upvote_num')
    def upvote_num(self):
        """获取答案赞同数量.

        :return: 答案赞同数量
        :rtype: int
        """
        return int(self.soup.find(
            'div', class_='zm-item-vote-info')['data-votecount'])

    @property
    def upvoters(self):
        """获取答案点赞用户，返回生成器.

        :return: 点赞用户
        :rtype: Author.Iterable
        """
        self._make_soup()
        next_req = '/answer/' + str(self.aid) + '/voters_profile'
        while next_req != '':
            data = self._session.get(Zhihu_URL + next_req).json()
            next_req = data['paging']['next']
            for html in data['payload']:
                soup = BeautifulSoup(html)
                yield self._parse_author_soup(soup)

    @property
    @check_soup('_content')
    def content(self):
        """以处理过的Html代码形式返回答案内容.

        :return: 答案内容
        :rtype: str
        """
        answer_wrap = self.soup.find('div', id='zh-question-answer-wrap')
        content = answer_wrap.find('div', class_='zm-editable-content')
        content = answer_content_process(content)
        return content

    @property
    @check_soup('_creation_time')
    def creation_time(self):
        """获取答案创建时间

        :return: 答案创建时间
        :rtype: datetime.datetime
        """
        return datetime.fromtimestamp(int(self.soup.find(
                'div', class_='zm-item-answer')['data-created']))

    def save(self, filepath=None, filename=None, mode="html"):
        """保存答案为Html文档或markdown文档.

        :param str filepath: 要保存的文件所在的目录，
            不填为当前目录下以问题标题命名的目录, 设为"."则为当前目录。
        :param str filename: 要保存的文件名，
            不填则默认为 所在问题标题 - 答主名.html/md。
            如果文件已存在，自动在后面加上数字区分。
            **自定义文件名时请不要输入后缀 .html 或 .md。**
        :param str mode: 保存类型，可选 `html` 、 `markdown` 、 `md` 。
        :return: 无
        :rtype: None
        """
        if mode not in ["html", "md", "markdown"]:
            raise ValueError("`mode` must be 'html', 'markdown' or 'md',"
                             " got {0}".format(mode))
        file = get_path(filepath, filename, mode, self.question.title,
                        self.question.title + '-' + self.author.name)
        with open(file, 'wb') as f:
            if mode == "html":
                f.write(self.content.encode('utf-8'))
            else:
                import html2text
                h2t = html2text.HTML2Text()
                h2t.body_width = 0
                f.write(h2t.handle(self.content).encode('utf-8'))

    def _parse_author_soup(self, soup):
        from .author import Author

        author_tag = soup.find('div', class_='body')
        if author_tag.string is None:
            author_name = author_tag.div.a['title']
            author_url = author_tag.div.a['href']
            author_motto = author_tag.div.span.text
            photo_url = PROTOCOL + soup.a.img['src'].replace('_m', '_r')
            numbers_tag = soup.find_all('li')
            numbers = [int(re_get_number.match(x.get_text()).group(1))
                       for x in numbers_tag]
        else:
            author_url = None
            author_name = '匿名用户'
            author_motto = ''
            numbers = [None] * 4
            photo_url = None
        # noinspection PyTypeChecker
        return Author(author_url, author_name, author_motto, None,
                      numbers[2], numbers[3], numbers[0], numbers[1],
                      photo_url, session=self._session)

    @property
    def comments(self):
        """获取答案下的所有评论.

        :return: 答案下的所有评论，返回生成器
        :rtype: Comments.Iterable
        """
        from .author import Author
        from .comment import Comment
        r = self._session.get(Answer_Comment_Box_URL,
                              params='params='+json.dumps({'answer_id': self.aid, 'load_all': True}))
        soup = BeautifulSoup(r.content)
        comment_items = soup.find_all('div', class_='zm-item-comment')
        for comment_item in comment_items:
            comment_id = int(comment_item['data-id'])
            content = comment_item.find(
                'div', class_='zm-comment-content').text.replace('\n', '')
            upvote_num = int(comment_item.find('span', class_='like-num').em.text)
            time_string = comment_item.find('span', class_='date').text
            a_url, a_name, a_photo_url = parser_author_from_comment(comment_item)
            author_obj = Author(a_url, a_name, photo_url=a_photo_url,
                                session=self._session)
            yield Comment(comment_id, self, author_obj, upvote_num, content, time_string)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import print_function, division, unicode_literals
import unittest
import os
import json

from test_utils import TEST_DATA_PATH
from zhihu import Column, Post


class ColumnTest(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        url = 'http://zhuanlan.zhihu.com/xiepanda'
        file_path = os.path.join(TEST_DATA_PATH, 'column.json')
        with open(file_path, 'r') as f:
            soup = json.load(f)

        post_path = os.path.join(TEST_DATA_PATH, 'column_post.json')
        with open(post_path, 'r') as f:
            cls.post_json = json.load(f)

        cls.column = Column(url)
        cls.column.soup = soup
        cls.expected = {'name': '谢熊猫出没注意', 'follower_num': 76605,
                        'post_num': 69, 'post_author_id': 'xiepanda',
                        'post_title': ("为了做一个称职的吃货，他决定连着吃"
                                       "一百天转基因食物"),
                        'post_upvote_num': 963, 'post_comment_num': 199}

    def test_name(self):
        self.assertEqual(self.expected['name'], self.column.name)

    def test_folower_num(self):
        self.assertEqual(self.expected['follower_num'],
                         self.column.follower_num)

    def test_post_num(self):
        self.assertEqual(self.expected['post_num'], self.column.post_num)

    def test_parse_post_data(self):
        post = self.column._parse_post_data(self.post_json)
        self.assertEqual(self.expected['post_author_id'], post.author.id)
        self.assertEqual(self.expected['post_title'], post.title)
        self.assertEqual(self.expected['post_upvote_num'], post.upvote_num)
        self.assertEqual(self.expected['post_comment_num'], post.comment_num)

    def test_posts(self):
        ps = self.column.posts
        post = next(ps)
        self.assertTrue(isinstance(post, Post))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import enum

match = {
    'ANSWER_QUESTION': 'member_answer_question',
    'UPVOTE_ANSWER': 'member_voteup_answer',
    'ASK_QUESTION': 'member_ask_question',
    'FOLLOW_QUESTION': 'member_follow_question',
    'UPVOTE_POST': 'member_voteup_article',
    'FOLLOW_COLUMN': 'member_follow_column',
    'FOLLOW_TOPIC': 'member_follow_topic',
    'PUBLISH_POST': 'member_create_article'
}

reverse_match = {v: k for k, v in match.items()}


class ActType(enum.Enum):

    """用于表示用户动态的类型.

    :常量说明:
        =============== ============== ======== ==================
        常量名          说明           提供属性 属性类型
        =============== ============== ======== ==================
        ANSWER_QUESTION 回答了一个问题 answer   :class:`.Answer`
        UPVOTE_ANSWER   赞同了一个回答 answer   :class:`.Answer`
        ASK_QUESTION    提出了一个问题 question :class:`.Question`
        FOLLOW_QUESTION 关注了一个问题 question :class:`.Question`
        UPVOTE_POST     赞同了一篇文章 post     :class:`.Post`
        FOLLOW_COLUMN   关注了一个专栏 column   :class:`.Column`
        FOLLOW_TOPIC    关注了一个话题 topic    :class:`.Topic`
        PUBLISH_POST    发表了一篇文章 post     :class:`.Post`
        =============== ============== ======== ==================

    """

    ANSWER_QUESTION = 1
    UPVOTE_ANSWER = 2
    ASK_QUESTION = 4
    FOLLOW_QUESTION = 8
    UPVOTE_POST = 16
    FOLLOW_COLUMN = 32
    FOLLOW_TOPIC = 64
    PUBLISH_POST = 128

    @classmethod
    def from_str(cls, div_class):
        return cls.__getattr__(reverse_match[div_class])

    def __str__(self):
        return match[self.name]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import re
import ast

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def extract_version():
    with open('zhihu/__init__.py', 'rb') as f_version:
        ast_tree = re.search(
            r'__version__ = (.*)',
            f_version.read().decode('utf-8')
        ).group(1)
        if ast_tree is None:
            raise RuntimeError('Cannot find version information')
        return str(ast.literal_eval(ast_tree))


with open('README.rst', 'rb') as f_readme:
    readme = f_readme.read().decode('utf-8')

packages = ['zhihu']

version = extract_version()

setup(
    name='zhihu-py3',
    version=version,
    keywords=['zhihu', 'network', 'spider', 'html'],
    description='Zhihu UNOFFICIAL API library in python3, '
                'with help of bs4, lxml, requests and html2text.',
    long_description=readme,

    author='7sDream',
    author_email='didislover@gmail.com',
    license='MIT',

    url='https://github.com/7sDream/zhihu-py3',
    download_url='https://github.com/7sDream/zhihu-py3',

    install_requires=[
        'beautifulsoup4',
        'requests',
        'html2text'
    ],
    extras_require={
        'lxml': ['lxml']
    },
    packages=packages,

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
